from openai import OpenAI
import os
import re
import sys
from parser import parse_file as parser
from parser import FileLine


class TranslationBatch:
    def __init__(self, lines):
        self._lines = lines

    def __len__(self):
        return len(self._lines)

    def __str__(self):
        return "\n".join([line.to_translated for line in self._lines.values()])

    def split(self):
        half = len(self._lines) // 2
        return (
            self.__class__(self._lines[:half]), 
            self.__class__(self._lines[half:])
        )


class TranslationError(Exception):
    """Generic translation error"""


class MissingLine(TranslationError):
    """There is missing line in file, it might be a need to re-translate."""


class TranslatableFile:
    def __init__(self, file_path) -> None:
        self._file_path = file_path
        self._parsed = parser(file_path)
        self._translated = {
            line.line_number: line 
            for line in self._parsed 
            if not line.to_translate
        }
        self._to_translate = {
            line.line_number: line 
            for line in self._parsed 
            if line.to_translate
        }
        self._number_of_lines = len(self._translated) + 2 * len(self._to_translate)

    def __iter__(self):
        return iter(sorted(self._translated.values(), key=lambda l: l.line_number))

    @property
    def path(self):
        return self._file_path

    @property
    def is_translated(self) -> bool:
        return len(self._translated) == self._number_of_lines

    def get_translation_batch(self):
        return TranslationBatch(self._to_translate)

    def update_translation(self, translated_lines):
        for line in translated_lines:
            index, rest = re.search("(\d*)\s*(.*)", line).groups()
            if not index:
                continue
            index = int(index)
            try:
                to_translate = self._to_translate.pop(index)
            except KeyError as e:
                raise MissingLine()
            self._translated[index + 1] = to_translate.from_translation(index + 1, rest)
            self._translated[index] = to_translate

    def save(self):
        with open(self._file_path, "w") as file:
            file.writelines([line.output_line for line in self])


class Translator:
    MESSAGE_TEMPLATE = (
        "You are a translator for an adult explicit video game. Translate lines I'm going to send you in next "
        "message to {}. Preserve numbers at the beginning of each line. Do not translate things inside square brackets. Make the translated text similar length as the original."
    )

    def __init__(self, openai_client, target_language):
        self._client = openai_client
        self._language = target_language

    def translate_file(self, file):
        batches = [file.get_translation_batch()]
        while batches:
            current_batch = batches.pop()
            batch_size = len(current_batch)

            if not batch_size:
                print("Empty batch, going further.")
                continue

            print("About to translate batch! Batch size: ", len(current_batch))
            completion = self._client.chat.completions.create(
                messages=[
                    {
                        "role": "user",
                        "content": self.MESSAGE_TEMPLATE.format(self._language)
                    },
                    {
                        "role": "user",
                        "content": str(current_batch)
                    }
                ],
                model="gpt-4o-mini"
            )

            result = completion.choices[0].message.content.splitlines()
            file.update_translation(result)

            if batches:
                continue
            elif not file.is_translated:
                new_batch = file.get_translation_batch()
                batches.append(new_batch)
            else:
                break


def translate_directory(base_dir, target_languages):
    for target_language, lang_dir in target_languages.items():
        print(f"Translating files to {target_language} in directory: {lang_dir}")
        for root, _, files in os.walk(lang_dir):
            for file in files:
                if file.endswith(".rpy"):
                    file_path = os.path.join(root, file)
                    translate_file(file_path, target_language)


def translate_file(file_path, translator, errors):
    file = TranslatableFile(file_path)
    try:
        print("Translating file: ", file.path)
        translator.translate_file(file)
    except TranslationError:
        errors.append(file)
    else:
        file.save()


def main():
    base_dir = sys.argv[1]
    language = sys.argv[2]
    ignored_words = sys.argv[3:]

    api_key = os.environ["OPENAI_API_KEY"]
    client = OpenAI(api_key=api_key)
    translator = Translator(client, language)

    errors = []

    print("Start translation")
    try:
        if os.path.isfile(base_dir):
            print(f"Translate single file: {base_dir}")
            translate_file(base_dir, translator, errors)
            print("Done!")
            return

        for root, _, files in os.walk(base_dir):
            for filename in files:
                if filename.endswith(".rpy"):
                    file_path = os.path.join(root, filename)
                    translate_file(file_path, translator, errors)
    except KeyboardInterrupt:
        print("Aborting.")

    for error_file in errors:
        print("Could't translate:", error_file.path)

    print("Done!")


if __name__ == "__main__":
    main()
