import re
import sys
from dataclasses import dataclass
from enum import Enum, auto
from itertools import zip_longest

STARTING_COMMENT = "    # "
STARTING_UI_SEQ = "    old "


class LineType(Enum):
    DIALOGUE = auto()
    UI = auto()
    NORMAL = auto()


@dataclass
class FileLine:
    line_number: int
    prefix: str
    content: str
    original: str
    to_translate: bool
    translated: bool = False
    wrapper_prefix: str = ''
    wrapper_suffix: str = ''
    line_type: LineType = LineType.NORMAL

    @property
    def output_prefix(self):
        prefix = self.prefix
        if self.line_type is LineType.UI and self.translated:
            prefix = prefix.replace("old", "new")
        return prefix

    @property
    def output_line(self):
        if self.original == "\n":
            return self.original

        match self.line_type:
            case LineType.UI:
                return f"    {self.output_prefix} \"{self.content}\"\n"
            case LineType.DIALOGUE:
                prefix = "# " if self.to_translate else ""
                prefix += self.prefix if self.prefix == "" else self.prefix + " "
                escaped_content = self.content.replace('"', r'\"')
                return f"    {prefix}\"{self.wrapper_prefix}{escaped_content}{self.wrapper_suffix}\"\n"
            case _:
                return self.content

    @property
    def match_original(self):
        return self.output_line == self.original

    @property
    def to_translated(self):
        return str(self.line_number) + " " + self.content

    def from_translation(self, index, content):
        return self.__class__(index, self.prefix, content, self.original, False, True, self.wrapper_prefix, self.wrapper_suffix, self.line_type)



def parse_dialogue_line(line, number) -> FileLine:
    original = str(line)
    line = line[len(STARTING_COMMENT):]
    line_start = 0
    old_line = str(line)
    if line.startswith('"'):
        prefix = ''
    else:
        line_start = line.index('"')
        prefix = line[:line_start].strip()
    
    line = line.strip()[line_start + 1:-1]

    wrapper_prefix = ''
    wrapper_suffix = ''

    if line.startswith("{"):
        start, end = line.index("}") + 1, len(line) - (line[::-1].index("{") + 1)
        wrapper_prefix = line[:start]
        wrapper_suffix = line[end:]
        line = line[start:end]

#    if "Despite a trio smoking" in line:
#        raise Exception(old_line, line, prefix, wrapper_prefix, wrapper_suffix)

    return FileLine(number, prefix, line, original, True, False, wrapper_prefix, wrapper_suffix, line_type=LineType.DIALOGUE)


def parse_ui_line(line, number) -> FileLine:
    original = str(line)
    line = line[len(STARTING_UI_SEQ):].strip()
    prefix = "old"
    line = line[1:-1]
    return FileLine(number, prefix, line, original, True, False, line_type=LineType.UI)


def dummy_parser(line, number) -> FileLine:
    return FileLine(number, '', line, line, False, True, line_type=LineType.NORMAL)


def parse_file(path):
    with open(path, 'r', encoding='utf-8') as file:
        lines = file.readlines()

    parsed = []
    skip_next_line = False
    not_parse_next = False

    for (index, (line, next_line)) in enumerate(zip_longest(lines, lines[1:])):
        if skip_next_line:
            skip_next_line = False
            continue

        if not_parse_next:
            parser = dummy_parser
            not_parse_next = False
        elif line.startswith(STARTING_COMMENT) and '"' in line and next_line is not None and re.search('".+"', next_line):
            parser = dummy_parser
            not_parse_next = True
        elif line.startswith(STARTING_COMMENT) and '"' in line:
            parser = parse_dialogue_line
            skip_next_line = True
        elif line.startswith(STARTING_UI_SEQ) and next_line is not None and re.search('".+"', next_line):
            parser = dummy_parser
            not_parse_next = True
        elif line.startswith(STARTING_UI_SEQ):
            parser = parse_ui_line
            skip_next_line = True
        else:
            parser = dummy_parser
        
        parsed.append(parser(line, index))
    return parsed


def main():
    filepath = sys.argv[1]
    result = parse_file(filepath)
    for r in result:
        if not r.match_original:
            print(repr(r.original), repr(r.output_line), r.line_type)


if __name__ == "__main__":
    main()

